@@ -3,16 +3,17 @@ use std::collections::{BTreeMap, BTreeSet, btree_map::Entry};
 use crate::clarity::{analysis::AnalysisDatabase, database::ClarityBackingStore};
 use crate::clarity::analysis::ContractAnalysis;
 use crate::clarity::ast::ContractAST;
-use crate::clarity::contexts::{ContractContext, GlobalContext};
+use crate::clarity::contexts::{ContractContext, GlobalContext, CallStack, Environment, LocalContext};
 use crate::clarity::contracts::Contract;
 use crate::clarity::costs::{LimitedCostTracker, ExecutionCost};
 use crate::clarity::database::{Datastore, NULL_HEADER_DB};
 use crate::clarity::diagnostic::Diagnostic;
-use crate::clarity::eval_all;
+use crate::clarity::{eval_all, eval};
 use crate::clarity::types::{self, PrincipalData, StandardPrincipalData, QualifiedContractIdentifier};
 use crate::clarity::util::StacksAddress;
 use crate::clarity::{analysis, ast};
 use crate::clarity::events::*;
+use crate::clarity::coverage::TestCoverageReport;
 use crate::repl::{CostSynthesis, ExecutionResult};
 use serde_json::Value;
 
@@ -44,11 +45,12 @@ impl ClarityInterpreter {
         &mut self,
         snippet: String,
         contract_identifier: QualifiedContractIdentifier,
-        cost_track: bool
+        cost_track: bool,
+        coverage_reporter: Option<TestCoverageReport>
     ) -> Result<ExecutionResult, (String, Option<Diagnostic>)> {
         let mut ast = self.build_ast(contract_identifier.clone(), snippet.clone())?;
         let analysis = self.run_analysis(contract_identifier.clone(), &mut ast)?;
-        let result = self.execute(contract_identifier, &mut ast, snippet, analysis, cost_track)?;
+        let result = self.execute(contract_identifier, &mut ast, snippet, analysis, cost_track, coverage_reporter)?;
 
         // todo: instead of just returning the value, we should be returning:
         // - value
@@ -119,6 +121,7 @@ impl ClarityInterpreter {
         snippet: String,
         contract_analysis: ContractAnalysis,
         cost_track: bool,
+        coverage_reporter: Option<TestCoverageReport>,
     ) -> Result<ExecutionResult, (String, Option<Diagnostic>)> {
 
         let mut execution_result = ExecutionResult::default();
@@ -128,6 +131,8 @@ impl ClarityInterpreter {
         let mut accounts_to_credit = vec![];
         let mut contract_context = ContractContext::new(contract_identifier.clone());
         let value = {
+            let tx_sender: PrincipalData = self.tx_sender.clone().into();
+
             let mut conn = self.datastore.as_clarity_db(&NULL_HEADER_DB);
             let cost_tracker = if cost_track {
                 LimitedCostTracker::new(false, BLOCK_LIMIT_MAINNET.clone(), &mut conn).unwrap()
@@ -135,10 +140,43 @@ impl ClarityInterpreter {
                 LimitedCostTracker::new_free()
             };
             let mut global_context = GlobalContext::new(false, conn, cost_tracker);
+            global_context.coverage_reporting = coverage_reporter;
             global_context.begin();
 
-            let result = global_context
-                .execute(|g| eval_all(&contract_ast.expressions, &mut contract_context, g));
+            let result = global_context.execute(|g| {
+                // If we have more than one instruction
+                if contract_ast.expressions.len() == 1 && !snippet.contains("(define-") {                    
+                    let context = LocalContext::new();                
+                    let mut call_stack = CallStack::new();
+                    let mut env = Environment::new(
+                        g, &mut contract_context, &mut call_stack, Some(tx_sender.clone()), Some(tx_sender.clone()));
+
+                    let result = match contract_ast.expressions[0].expr {
+                        List(ref expression) => match expression[0].expr {
+                            Atom(ref name) if name.to_string() == "contract-call?" => {
+                                let contract_identifier = match expression[1].match_literal_value().unwrap().clone().expect_principal() {
+                                    PrincipalData::Contract(contract_identifier) => contract_identifier,
+                                    _ => unreachable!()
+                                };
+                                let method = expression[2].match_atom().unwrap().to_string();
+                                let mut args = vec![];
+                                for arg in expression[3..].iter() {
+                                    let evaluated_arg = eval(arg, &mut env, &context)?;
+                                    args.push(SymbolicExpression::atom_value(evaluated_arg));
+                                }
+                                env.execute_contract(&contract_identifier, &method, &args, false).unwrap()
+                            },
+                            _ => eval(&contract_ast.expressions[0], &mut env, &context).unwrap()
+                        },
+                        _ => eval(&contract_ast.expressions[0], &mut env, &context).unwrap()
+                    };
+                    Ok(Some(result))    
+                } else {
+                    eval_all(&contract_ast.expressions, &mut contract_context, g)
+                }
+            });
+
+            execution_result.coverage = global_context.coverage_reporting.take();
 
             let value = match result {
                 Ok(Some(value)) => format!("{}", value),
@@ -217,7 +255,7 @@ impl ClarityInterpreter {
 
                     functions.insert(name.to_string(), args);
                 }
-                execution_result.contract = Some((format!("{}", contract_identifier), functions));
+                execution_result.contract = Some((format!("{}", contract_identifier), functions, contract_ast.clone()));
 
                 for defined_trait in contract_context.defined_traits.iter() {}
 
