@@ -1,14 +1,17 @@
 use super::{ClarityInterpreter, ExecutionResult};
-use crate::clarity::diagnostic::Diagnostic;
+use crate::{clarity::diagnostic::Diagnostic, repl::settings::InitialContract};
 use crate::clarity::docs::{make_api_reference, make_define_reference, make_keyword_reference};
 use crate::clarity::functions::define::DefineFunctions;
 use crate::clarity::functions::NativeFunctions;
 use crate::clarity::types::{PrincipalData, StandardPrincipalData, QualifiedContractIdentifier};
 use crate::clarity::util::StacksAddress;
+use crate::clarity::ast::ContractAST;
 use crate::clarity::variables::NativeVariables;
+use crate::clarity::coverage::{CoverageReporter, TestCoverageReport};
 use crate::contracts::{POX_CONTRACT, BNS_CONTRACT, COSTS_CONTRACT};
 use ansi_term::{Colour, Style};
 use std::collections::{HashMap, BTreeSet, VecDeque, BTreeMap};
+use std::sync::{Arc, Mutex};
 use serde_json::Value;
 
 #[cfg(feature = "cli")]
@@ -17,6 +20,8 @@ use prettytable::{Table, Row, Cell};
 use super::SessionSettings;
 use super::settings::InitialLink;
 
+#[cfg(feature = "wasm")]
+use reqwest_wasm as reqwest;
 enum Command {
     LoadLocalContract(String),
     LoadDeployContract(String),
@@ -30,10 +35,12 @@ enum Command {
 pub struct Session {
     session_id: u32,
     started_at: u32,
-    settings: SessionSettings,
+    pub settings: SessionSettings,
     contracts: BTreeMap<String, BTreeMap<String, Vec<String>>>,
+    pub asts: BTreeMap<QualifiedContractIdentifier, ContractAST>,
     interpreter: ClarityInterpreter,
     api_reference: HashMap<String, String>,
+    pub coverage_reports: Vec<TestCoverageReport>,
 }
 
 impl Session {
@@ -51,13 +58,15 @@ impl Session {
             session_id: 0,
             started_at: 0,
             settings,
+            asts: BTreeMap::new(),
             contracts: BTreeMap::new(),
             interpreter: ClarityInterpreter::new(tx_sender),
             api_reference: build_api_reference(),
+            coverage_reports: vec![],
         }
     }
 
-    fn retrieve_contract(&mut self, link: &InitialLink) -> Result<(String, BTreeSet<String>), String> {
+    async fn retrieve_contract(&mut self, link: &InitialLink) -> Result<(String, BTreeSet<String>), String> {
         let contract_id = &link.contract_id;
         let components: Vec<&str> = contract_id.split('.').collect();
         let contract_deployer = components.first().expect("");
@@ -71,12 +80,6 @@ impl Session {
             }
         };
 
-        #[derive(Deserialize, Debug)]
-        struct Contract {
-            source: String,
-            publish_height: u32,
-        }
-
         let request_url = format!(
             "{host}/v2/contracts/source/{addr}/{name}?proof=0",
             host = stacks_node_addr,
@@ -84,17 +87,15 @@ impl Session {
             name = contract_name
         );
 
-        let response: Contract = reqwest::blocking::get(&request_url)
-            .expect("Unable to retrieve contract")
-            .json()
-            .expect("Unable to parse contract");
+        let response = fetch_contract(request_url).await;
+        
         let code = response.source.to_string();
         let deps = self.interpreter.detect_dependencies(contract_id.to_string(), code.clone())
             .unwrap();
         Ok((code, deps))
     }
 
-    pub fn resolve_link(&mut self, link: &InitialLink) -> Result<Vec<(String, String, Vec<String>)>, String> {
+    pub async fn resolve_link(&mut self, link: &InitialLink) -> Result<Vec<(String, String, Vec<String>)>, String> {
         let mut resolved_link = Vec::new();
 
         let mut handled: HashMap<String, String> = HashMap::new();
@@ -113,6 +114,7 @@ impl Session {
                 Some(entry) => (entry.clone(), BTreeSet::new()),
                 None => {
                     let (contract_code, deps) = self.retrieve_contract(&initial_link)
+                        .await
                         .expect("Unable to get contract");
                     handled.insert(contract_id.to_string(), contract_code.clone());
                     (contract_code, deps)
@@ -141,25 +143,69 @@ impl Session {
         Ok(resolved_link)
     }
 
+    #[cfg(not(feature = "wasm"))] 
     pub fn start(&mut self) -> String {
         let mut output = Vec::<String>::new();
 
+        if !self.settings.include_boot_contracts.is_empty() {
+            let default_tx_sender = self.interpreter.get_tx_sender();
+
+            let boot_testnet_address = "ST000000000000000000002AMW42H";
+            let boot_testnet_deployer = PrincipalData::parse_standard_principal(&boot_testnet_address)
+                .expect("Unable to parse deployer's address");
+
+            self.interpreter.set_tx_sender(boot_testnet_deployer);
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
+
+            let boot_mainnet_address = "SP000000000000000000002Q6VF78";
+            let boot_mainnet_deployer = PrincipalData::parse_standard_principal(&boot_mainnet_address)
+                .expect("Unable to parse deployer's address");            
+            self.interpreter.set_tx_sender(boot_mainnet_deployer);
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
+            self.interpreter.set_tx_sender(default_tx_sender);
+        }
+
+        let mut linked_contracts = Vec::new();
+
         if self.settings.initial_links.len() > 0 {
             let initial_links = self.settings.initial_links.clone();
             let default_tx_sender = self.interpreter.get_tx_sender();
 
-            let mut all_contracts = Vec::new();
             let mut indexed = BTreeSet::new();
+
+            let rt = tokio::runtime::Runtime::new().unwrap();
             for link in initial_links.iter() {
-                let contracts = self.resolve_link(link).unwrap();
+                let contracts = rt.block_on(async { self.resolve_link(link).await.unwrap() });
                 for (contract_id, code, _) in contracts.into_iter() {
                     if !indexed.contains(&contract_id) {
                         indexed.insert(contract_id.clone());
-                        all_contracts.push((contract_id, code));
+                        linked_contracts.push((contract_id, code));
                     }
                 }
             }
-            for (contract_id, code) in all_contracts.into_iter() {
+            for (contract_id, code) in linked_contracts.iter() {
                 let components: Vec<&str> = contract_id.split('.').collect();
                 let contract_deployer = components.first().expect("");
                 let contract_name = components.last().expect("");
@@ -170,14 +216,14 @@ impl Session {
                 };
 
                 self.interpreter.set_tx_sender(deployer);
-                match self.formatted_interpretation(code.to_string(), Some(contract_name.to_string()), true) {
+                match self.formatted_interpretation(code.to_string(), Some(contract_name.to_string()), true, None) {
                     Ok(_) => {},
                     Err(ref mut result) => output.append(result),
                 };
+
             }
 
             self.interpreter.set_tx_sender(default_tx_sender);
-            output.push(blue!("Initial links"));
         }
 
         if self.settings.initial_accounts.len() > 0 {
@@ -201,32 +247,6 @@ impl Session {
             }
         }
 
-        if self.settings.include_boot_contracts {
-            let default_tx_sender = self.interpreter.get_tx_sender();
-
-            let boot_testnet_address = "ST000000000000000000002AMW42H";
-            let boot_testnet_deployer = PrincipalData::parse_standard_principal(&boot_testnet_address)
-                .expect("Unable to parse deployer's address");            
-            self.interpreter.set_tx_sender(boot_testnet_deployer);
-            self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false)
-                .expect("Unable to deploy POX");
-            self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false)
-                .expect("Unable to deploy BNS");
-            self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false)
-                .expect("Unable to deploy COSTS");
-            let boot_mainnet_address = "SP000000000000000000002Q6VF78";
-            let boot_mainnet_deployer = PrincipalData::parse_standard_principal(&boot_mainnet_address)
-                .expect("Unable to parse deployer's address");            
-            self.interpreter.set_tx_sender(boot_mainnet_deployer);
-            self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false)
-                .expect("Unable to deploy POX");
-            self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false)
-                .expect("Unable to deploy BNS");
-            self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false)
-                .expect("Unable to deploy COSTS");
-            self.interpreter.set_tx_sender(default_tx_sender);
-        }
-
         if self.settings.initial_contracts.len() > 0 {
             let mut initial_contracts = self.settings.initial_contracts.clone();
             let default_tx_sender = self.interpreter.get_tx_sender();
@@ -241,13 +261,33 @@ impl Session {
                 };
 
                 self.interpreter.set_tx_sender(deployer);
-                match self.formatted_interpretation(contract.code, contract.name, true) {
+                match self.formatted_interpretation(contract.code, contract.name, true, Some("Deployment".into())) {
                     Ok(_) => {},
                     Err(ref mut result) => output.append(result),
                 };
             }
             self.interpreter.set_tx_sender(default_tx_sender);
-            output.push(blue!("Initialized contracts"));
+        }
+
+        for (contract_id, code) in linked_contracts.into_iter() {
+            let components: Vec<&str> = contract_id.split('.').collect();
+            let contract_deployer = components.first().expect("");
+            let contract_name = components.last().expect("");
+
+            let deployer = {
+                PrincipalData::parse_standard_principal(&contract_deployer)
+                    .expect("Unable to parse deployer's address")
+            };
+            self.settings.initial_contracts.push(InitialContract {
+                code: code.to_string(),
+                path: "".into(),
+                name: Some(contract_id.to_string()),
+                deployer: Some(deployer.to_string())
+            });
+        }
+
+        if !self.settings.initial_contracts.is_empty() {
+            output.push(blue!("Contracts"));
             self.get_contracts(&mut output);
         }
 
@@ -259,32 +299,153 @@ impl Session {
         output.join("\n")
     }
 
-    pub fn check(&mut self) -> Result<(), String> {
+    #[cfg(feature = "wasm")] 
+    pub async fn start_wasm(&mut self) -> String {
         let mut output = Vec::<String>::new();
 
-        if self.settings.include_boot_contracts {
+        if !self.settings.include_boot_contracts.is_empty() {
             let default_tx_sender = self.interpreter.get_tx_sender();
 
             let boot_testnet_address = "ST000000000000000000002AMW42H";
             let boot_testnet_deployer = PrincipalData::parse_standard_principal(&boot_testnet_address)
-                .expect("Unable to parse deployer's address");            
+                .expect("Unable to parse deployer's address");
+
             self.interpreter.set_tx_sender(boot_testnet_deployer);
-            self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false)
-                .expect("Unable to deploy POX");
-            self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false)
-                .expect("Unable to deploy BNS");
-            self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false)
-                .expect("Unable to deploy COSTS");
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
+
             let boot_mainnet_address = "SP000000000000000000002Q6VF78";
             let boot_mainnet_deployer = PrincipalData::parse_standard_principal(&boot_mainnet_address)
                 .expect("Unable to parse deployer's address");            
             self.interpreter.set_tx_sender(boot_mainnet_deployer);
-            self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false)
-                .expect("Unable to deploy POX");
-            self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false)
-                .expect("Unable to deploy BNS");
-            self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false)
-                .expect("Unable to deploy COSTS");
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
+            self.interpreter.set_tx_sender(default_tx_sender);
+        }
+
+        let mut linked_contracts = Vec::new();
+
+        if self.settings.initial_links.len() > 0 {
+            let initial_links = self.settings.initial_links.clone();
+            let default_tx_sender = self.interpreter.get_tx_sender();
+
+            let mut indexed = BTreeSet::new();
+            for link in initial_links.iter() {
+                let contracts = self.resolve_link(link).await.unwrap();
+                for (contract_id, code, _) in contracts.into_iter() {
+                    if !indexed.contains(&contract_id) {
+                        indexed.insert(contract_id.clone());
+                        linked_contracts.push((contract_id, code));
+                    }
+                }
+            }
+            for (contract_id, code) in linked_contracts.iter() {
+                let components: Vec<&str> = contract_id.split('.').collect();
+                let contract_deployer = components.first().expect("");
+                let contract_name = components.last().expect("");
+
+                let deployer = {
+                    PrincipalData::parse_standard_principal(&contract_deployer)
+                        .expect("Unable to parse deployer's address")
+                };
+
+                self.interpreter.set_tx_sender(deployer);
+                match self.formatted_interpretation(code.to_string(), Some(contract_name.to_string()), true, None) {
+                    Ok(_) => {},
+                    Err(ref mut result) => output.append(result),
+                };
+            }
+
+            self.interpreter.set_tx_sender(default_tx_sender);
+            self.get_contracts(&mut output);
+        }
+
+
+        for (contract_id, code) in linked_contracts.iter() {
+            let components: Vec<&str> = contract_id.split('.').collect();
+            let contract_deployer = components.first().expect("");
+            let contract_name = components.last().expect("");
+
+            let deployer = {
+                PrincipalData::parse_standard_principal(&contract_deployer)
+                    .expect("Unable to parse deployer's address")
+            };
+            self.settings.initial_contracts.push(InitialContract {
+                code: code.to_string(),
+                path: "".into(),
+                name: Some(contract_id.to_string()),
+                deployer: Some(deployer.to_string())
+            });
+        }
+
+        if !self.settings.initial_contracts.is_empty() {
+            output.push(blue!("Contracts"));
+            self.get_contracts(&mut output);
+        }
+
+        output.join("\n")
+    }
+
+    pub fn check(&mut self) -> Result<(), String> {
+        let mut output = Vec::<String>::new();
+
+        if !self.settings.include_boot_contracts.is_empty() {
+            let default_tx_sender = self.interpreter.get_tx_sender();
+
+            let boot_testnet_address = "ST000000000000000000002AMW42H";
+            let boot_testnet_deployer = PrincipalData::parse_standard_principal(&boot_testnet_address)
+                .expect("Unable to parse deployer's address");
+
+            self.interpreter.set_tx_sender(boot_testnet_deployer);
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
+
+            let boot_mainnet_address = "SP000000000000000000002Q6VF78";
+            let boot_mainnet_deployer = PrincipalData::parse_standard_principal(&boot_mainnet_address)
+                .expect("Unable to parse deployer's address");            
+            self.interpreter.set_tx_sender(boot_mainnet_deployer);
+            if self.settings.include_boot_contracts.contains(&"pox".to_string()) {
+                self.formatted_interpretation(POX_CONTRACT.to_string(), Some("pox".to_string()), false, None)
+                    .expect("Unable to deploy POX");
+            }
+            if self.settings.include_boot_contracts.contains(&"bns".to_string()) {
+                self.formatted_interpretation(BNS_CONTRACT.to_string(), Some("bns".to_string()), false, None)
+                    .expect("Unable to deploy BNS");
+            }
+            if self.settings.include_boot_contracts.contains(&"costs".to_string()) {
+                self.formatted_interpretation(COSTS_CONTRACT.to_string(), Some("costs".to_string()), false, None)
+                    .expect("Unable to deploy COSTS");
+            }
             self.interpreter.set_tx_sender(default_tx_sender);
         }
 
@@ -323,7 +484,7 @@ impl Session {
                 };
 
                 self.interpreter.set_tx_sender(deployer);
-                match self.formatted_interpretation(contract.code, contract.name, true) {
+                match self.formatted_interpretation(contract.code, contract.name, true, Some("Deployment".into())) {
                     Ok(_) => {},
                     Err(ref mut result) => output.append(result),
                 };
@@ -353,7 +514,7 @@ impl Session {
             cmd if cmd.starts_with("::advance_chain_tip") => self.parse_and_advance_chain_tip(&mut output, cmd),
 
             snippet => {
-                let mut result = match self.formatted_interpretation(snippet.to_string(), None, true) {
+                let mut result = match self.formatted_interpretation(snippet.to_string(), None, true, None) {
                     Ok((result, _)) => result,
                     Err(result) => result,
                 };
@@ -368,15 +529,16 @@ impl Session {
         snippet: String,
         name: Option<String>,
         cost_track: bool,
+        test_name: Option<String>,
     ) -> Result<(Vec<String>, ExecutionResult), Vec<String>> {
         let light_red = Colour::Red.bold();
 
-        let result = self.interpret(snippet.to_string(), name, cost_track);
+        let result = self.interpret(snippet.to_string(), name, cost_track, test_name);
         let mut output = Vec::<String>::new();
 
         match result {
             Ok(result) => {
-                if let Some((ref contract_name, _)) = result.contract {
+                if let Some((ref contract_name, _, _)) = result.contract {
                     let snippet = format!("→ .{} contract successfully stored. Use (contract-call? ...) for invoking the public functions:", contract_name.clone());
                     output.push(green!(snippet));
                 }
@@ -449,13 +611,22 @@ impl Session {
         snippet: String,
         name: Option<String>,
         cost_track: bool,
+        test_name: Option<String>,
     ) -> Result<ExecutionResult, (String, Option<Diagnostic>)> {
+
         let contract_name = match name {
             Some(name) => name,
             None => format!("contract-{}", self.contracts.len()),
         };
         let first_char = contract_name.chars().next().unwrap();
 
+        let report = if let Some(test_name) = test_name {
+            let coverage = TestCoverageReport::new(test_name.into());
+            Some(coverage)
+        } else {
+            None
+        };
+
         // Kludge for handling fully qualified contract_id vs sugared syntax
         let contract_identifier = if first_char.to_string() == "S" {
             QualifiedContractIdentifier::parse(&contract_name).unwrap()
@@ -465,10 +636,14 @@ impl Session {
             QualifiedContractIdentifier::parse(&id).unwrap()
         };
 
-        match self.interpreter.run(snippet, contract_identifier.clone(), cost_track) {
+        match self.interpreter.run(snippet, contract_identifier.clone(), cost_track, report) {
             Ok(result) => {
-                if let Some((ref contract_identifier, ref contract)) = result.contract {
-                    self.contracts.insert(contract_identifier.clone(), contract.clone());
+                if let Some(ref coverage) = result.coverage {
+                    self.coverage_reports.push(coverage.clone());
+                }
+                if let Some((ref contract_identifier_str, ref contract, ref ast)) = result.contract {
+                    self.asts.insert(contract_identifier.clone(), ast.clone());
+                    self.contracts.insert(contract_identifier_str.clone(), contract.clone());
                 }
                 Ok(result)
             }
@@ -622,7 +797,7 @@ impl Session {
     #[cfg(feature = "cli")]
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
-        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true) {
+        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true, None) {
             Ok((output, result)) => (output, result.cost.clone()),
             Err(output) => (output, None),
         };
@@ -679,7 +854,7 @@ impl Session {
 
     #[cfg(feature = "cli")]
     fn get_contracts(&mut self, output:&mut Vec<String>) {
-        if self.settings.initial_contracts.len() > 0 {
+        if self.contracts.len() > 0 {
             let mut table = Table::new();
             table.add_row(row!["Contract identifier", "Public functions"]);
             let contracts = self.contracts.clone();
@@ -709,7 +884,7 @@ impl Session {
     #[cfg(not(feature = "cli"))]
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
-        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true) {
+        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true, None) {
             Ok((output, result)) => (output, result.cost.clone()),
             Err(output) => (output, None),
         };
@@ -732,10 +907,9 @@ impl Session {
 
     #[cfg(not(feature = "cli"))]
     fn get_contracts(&mut self, output:&mut Vec<String>) {
-        if self.settings.initial_contracts.len() > 0 {
-            let mut initial_contracts = self.contracts.clone();
-            for contract in initial_contracts.drain(..) {
-                output.push(format!("{}", contract.0));
+        for (contract_id, methods) in self.contracts.iter() {
+            if !contract_id.ends_with(".pox") && !contract_id.ends_with(".bns") && !contract_id.ends_with(".costs") {
+                output.push(format!("{}", contract_id));
             }
         }
     }
@@ -796,6 +970,22 @@ impl Session {
     }
 }
 
+#[derive(Deserialize, Debug, Default, Clone)]
+struct Contract {
+    source: String,
+    publish_height: u32,
+}
+
+async fn fetch_contract(request_url: String) -> Contract {
+    let response: Contract = reqwest::get(&request_url)
+        .await
+        .expect("Unable to retrieve contract")
+        .json()
+        .await
+        .expect("Unable to parse contract");
+    return response;
+}
+
 fn build_api_reference() -> HashMap<String, String> {
     let mut api_reference = HashMap::new();
     for func in NativeFunctions::ALL.iter() {
