@@ -1,7 +1,10 @@
+use crate::clarity::coverage::CoverageReporter;
+
 #[derive(Clone, Debug)]
 pub struct InitialContract {
     pub code: String,
     pub name: Option<String>,
+    pub path: String,
     pub deployer: Option<String>,
 }
 
@@ -23,7 +26,7 @@ pub struct Account {
 
 #[derive(Clone, Debug, Default)]
 pub struct SessionSettings {
-    pub include_boot_contracts: bool,
+    pub include_boot_contracts: Vec<String>,
     pub initial_links: Vec<InitialLink>,
     pub initial_contracts: Vec<InitialContract>,
     pub initial_accounts: Vec<Account>,
